package locationServer;

import java.io.IOException;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;


/*
 Start opp databasen i terminalvinduet med flgende kommandoer : If you have installed the Startup Item, use this command:

shell> cd /usr/local/mysql
shell> sudo ./bin/mysqld_safe
(Enter your password, if necessary)
(Press Control-Z)
shell> bg
(Press Control-D or enter "exit" to exit the shell)
*/

/**
 * @author Gerhard Wiese
 * 
 * The DataBaseReader handles all contact with the database in the Location package. 
 *
 */
public class DataBaseReader {
	private Connection con = null;
	private String url = "jdbc:mysql://localhost:3306/";
	private String dbName = "TestLoc";
	private String driver = "com.mysql.jdbc.Driver";
	private String userName = "root"; 
	private String password = "";
	static long euclid;
	
	/**
	 * Creates a connection to the specific database, defined by the variables sat i the {@link DataBaseReader} class,
	 * and for logging on to the MySQL database.
	 */
	public void connectToDB(){
		try {
			  Class.forName(driver).newInstance();
			  con = DriverManager.getConnection(url+dbName,userName,password);
			  System.out.println("Connected to the database");			  
			} 
		catch (Exception e) {
			  e.printStackTrace();
			}
	}
	
	/**
	 * Disconnects from the MySQL database. 
	 * 
	 * @throws SQLException
	 */
	public void dissconnectDB() throws SQLException{
		con.close();
		System.out.println("Dissconnected from database");
	}

	
	/**
	 * The method is used for finding the location in the MySQL database best matching the real
	 * values read in by the {@link NetworkReader} from the network card in the user equipment. 
	 * The method will trigger the {@link NetworkReader} to read information from the network card,
	 * as well as making sure this information is sorted by the {@link Calculator}.
	 * FindLocation() will handle all connection to the the database, and executing all necessary queries,
	 * as well as trigging the calculation of eucleadian distance by the calculator.
	 * The coordinates of the estimated location is stored in the variables , xcoord, ycoord and
	 * zcoord in the {@link DataBaseReader} class. 
	 * 
	 * @see Calculator
	 * @see NetworkReader
	 * @throws SQLException
	 * @throws IOException
	 * @throws InterruptedException
	 */
	public String findLocation(ArrayList<AccessPoint> accessPoints) throws SQLException, IOException, InterruptedException{
		ArrayList<Location> locations = new ArrayList<Location>();
		ArrayList<AccessPoint> aps = accessPoints;
		Statement fpStatement = con.createStatement();
		Statement partialStatement = con.createStatement();
		Statement accStatement = con.createStatement();
		String result;
		

		String fp1 = "(SELECT AccessPoint.id FROM AccessPoint WHERE AccessPoint.bssid= \""+accessPoints.get(0).getBssi()+"\"  OR AccessPoint.bssid= \""+accessPoints.get(1).getBssi()+"\" OR AccessPoint.bssid= \""+accessPoints.get(2).getBssi()+"\")";
		fpStatement.execute("SELECT FingerPrint2.loc_id, FingerPrint2.acc_id, COUNT(FingerPrint2.loc_id) AS Count FROM FingerPrint2 WHERE FingerPrint2.acc_id IN "+fp1+" GROUP BY FingerPrint2.loc_id");
		ResultSet fingerPrints = fpStatement.getResultSet();
			if(fingerPrints.next()==false){
			System.out.println("Du befinner deg utenfor systemets dekningsomrdet!");
			}
			else{
			System.out.println("Lokasjoner er funnet!");
			
			while(fingerPrints.next()){
				if(fingerPrints.getInt("Count") == 3){
					System.out.println("Funnet match med tre aksesspunkt");
					int id =  fingerPrints.getInt("loc_id");	
					Location loc = new Location();
					loc.setLocationId(id);
					loc.setRating(0);
					partialStatement.execute("SELECT FingerPrint2.rssi, FingerPrint2.acc_id FROM FingerPrint2 WHERE FingerPrint2.loc_id ="+id);
					ResultSet rssiResult = partialStatement.getResultSet();
					while(rssiResult.next()){
						AccessPoint mid = new AccessPoint();
						mid.setSsid("eduroam");
						mid.setRssi(rssiResult.getInt("rssi"));
						accStatement.executeQuery("SELECT AccessPoint.bssid From AccessPoint WHERE AccessPoint.id = "+rssiResult.getInt("acc_id"));
						ResultSet acc = accStatement.getResultSet();
						acc.first();
						mid.setBssi(acc.getString("bssid"));
						loc.AccessPoints.add(mid);
					}
					locations.add(loc);	
				}
			}
				if(locations.isEmpty()){
					System.out.println("Finner kun matcher med to aksesspunkt");
					fingerPrints.beforeFirst();
					while(fingerPrints.next()){
					if(fingerPrints.getInt("Count") == 2){
						int id =  fingerPrints.getInt("loc_id");	
						Location loc = new Location();
						loc.setLocationId(id);
						loc.setRating(0);
						partialStatement.execute("SELECT FingerPrint2.rssi, FingerPrint2.acc_id FROM FingerPrint2 WHERE FingerPrint2.loc_id ="+id);
						ResultSet rssiResult = partialStatement.getResultSet();
						while(rssiResult.next()){
							AccessPoint mid = new AccessPoint();
							//skrevet om ikke snn det skal vre
							mid.setSsid("eduroam");
							mid.setRssi(rssiResult.getInt("rssi"));
							accStatement.executeQuery("SELECT AccessPoint.bssid From AccessPoint WHERE AccessPoint.id = "+rssiResult.getInt("acc_id"));
							ResultSet acc = accStatement.getResultSet();
							acc.first();
							mid.setBssi(acc.getString("bssid"));
							loc.AccessPoints.add(mid);
						}
						locations.add(loc);
					}
					}
					if(locations.isEmpty()){
						System.out.println("fantes bare lokasjoner linket til et aksesspunkt");
						return "one Ap only";
					}
				}
				
			int finalLocation = MainSystem.calc.euclideanDistance(locations, aps);
			System.out.println("Lokasjonid = "+finalLocation);
			fpStatement.executeQuery("SELECT cordx, cordy, cordz FROM Location WHERE Location.id ="+finalLocation);
			ResultSet finalResult = fpStatement.getResultSet();
			finalResult.first();
			
			int x = finalResult.getInt("cordx");
			int y = finalResult.getInt("cordy");
			int z = finalResult.getInt("cordz");
			result = (Integer.toString(x)+" "+Integer.toString(y)+" "+Integer.toString(z));
			System.out.println("X kordinat = "+finalResult.getInt("cordx")+" Y kordinat = "+finalResult.getInt("cordy")+" Z kordinat = "+finalResult.getInt("cordz"));
			return result;
			}
			return "No location found you may be outside of the covered area";
		
	}
}