package Location;

import java.util.ArrayList;

/**
 * @author Gerhard Wiese
 * 
 * The Calculator class is responsible for doing calculations in the Location system.
 *
 */
public class Calculator {
 
/**
 * SortAccessPointsByRssi will sort the {@link ArrayList} of {@link AccessPoint} contained in the MainSystem class of the 
 * Location package. The AccessPoint with the strongest rssi value will be put into the first position in the ArrayList etc. 
 * 
 * @see AccessPoint
 * @see ArrayList 
 */
public void sortAccessPointsByRssi(){
		ArrayList<AccessPoint> mid = MainSystem.accessPoints;	
		for(int i=1; i< MainSystem.accessPoints.size(); i++){
			for(int j=0;j < i; j++){
				if(MainSystem.accessPoints.get(i).getRssi() < MainSystem.accessPoints.get(j).getRssi()){					
					AccessPoint save =  MainSystem.accessPoints.get(i);
					MainSystem.accessPoints.remove(i);
					MainSystem.accessPoints.add(j, save);
						break;
					}
				}
			}
		}	
 
 /**
  * euclideanDistance will calculate the euclidean distance for all the {@link Location} objects in the input {@link ArrayList}, 
  * compared to the the values stored in the accessPoints {@link ArrayList} in {@link MainSystem}. The calculation is done in 
  * reference to the Rssi values. Each {@link Location} object contains a {@link ArrayList} of {@link AccessPoint}. 
  * accessPoints in {@link MainSystem} also contain {@link AccessPoint} objects.
  * 
  * 
  * The method will return the locationid of the {@link Location} corresponding to the smallest euclidean distance. 
  * 
 * @param locations a {@link ArrayList} of {@link Location} objects. 
 * @return The id of the {@link Location} object corresponding to the smallest euclidean distance
 * @see Location
 * @see ArrayList
 * @see MainSystem
 * @see AccessPoint
 */
public int euclideanDistance(ArrayList<Location> locations){
	 int rssi1 = MainSystem.accessPoints.get(0).getRssi(); 
	 int rssi2 = MainSystem.accessPoints.get(1).getRssi();
	 int rssi3 = MainSystem.accessPoints.get(2).getRssi();
	 int rssitest1 = 0;
	 int rssitest2 = 0;
	 int rssitest3 = 0;
	 int location = 0;
	 double edistance = 1000;
	 for(int i=0; i<locations.size(); i++){ 
		 for(int j = 0; j<locations.get(i).AccessPoints.size(); j++){
			 if(MainSystem.accessPoints.get(0).getBssi().equals(locations.get(i).AccessPoints.get(j).getBssi())){
				 rssitest1 = locations.get(i).AccessPoints.get(j).getRssi();
			 }
			 else if(MainSystem.accessPoints.get(1).getBssi().equals(locations.get(i).AccessPoints.get(j).getBssi())){
				 rssitest2 = locations.get(i).AccessPoints.get(j).getRssi();
			 }
			 else if(MainSystem.accessPoints.get(2).getBssi().equals(locations.get(i).AccessPoints.get(j).getBssi())){
				 rssitest3 = locations.get(i).AccessPoints.get(j).getRssi();
			 }
		 }
		 double distance = Math.sqrt(Math.pow((rssi1-rssitest1),2)+Math.pow((rssi2-rssitest2),2)+Math.pow((rssi3-rssitest3),2));
		 if(distance<edistance){
			  edistance = distance; 
			  location = locations.get(i).getLocationId();
		  }
	 }
	return location;
	 
	 }
}
