package Location;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.ScrollPane;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyListener;
import java.awt.image.BufferedImage;
import java.awt.image.WritableRaster;
import java.io.File;
import java.io.IOException;
import java.sql.SQLException;
import java.awt.event.ActionEvent;
import javax.imageio.ImageIO;
import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.ImageIcon;
import javax.swing.Scrollable;


/**
 * @author Gerhard Wiese
 * 
 *The Gui class contains all information for creating, handling and altering 
 *the Gui for the location system while it is running in operational mode. 
 */
public class Gui{
	private int pixMeter = 15; 
	private int xStartPoint; 
	private int yStartPoint;
	private BufferedImage firstFloorImg;
	private BufferedImage secondFloorImg;
	private String firstFloor = "/Users/Gerhard/Documents/workspace/Location/src/Location/e1.jpg";
	private String secondFloor = "/Users/Gerhard/Documents/workspace/Location/src/Location/e2.jpg";
	private JFrame frame = new JFrame("FindMyLocation");
	private JImagePanel panel;
	private JFrame startFrame = new JFrame("Login");
	private JPanel pan = new JPanel();
	
	private JButton btn = new JButton("Find My Location");
	private JButton startBtn = new JButton("Start System");
	JTextField coords = new JTextField(40);
	ActionListener listner;
	
	/**
	 * The constructor will initialize the GUI and set a ActionListener 
	 * listening to the {@link MainSystem} of the Location package.
	 * 
	 * @see MainSystem
	 * @param listner An ActionListner used for listening to the buttons pressed in the Gui.
	 */
	public Gui(ActionListener listner){
		this.listner = listner;
	}
	
	/**
	 * Creates a small GUI used for starting up the system. The GUI will be displayed in the left 
	 * corner of the screen and will contain a button used for starting up the location service of the system.
	 */
	public void startUpGui(){
		startFrame.add(pan);
		pan.add(startBtn);
		startFrame.setVisible(true);
		startFrame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		startFrame.setSize(250,250);
		startBtn.addActionListener(listner);
	}
	
	/**
	 * The method takes in three parameters that resembles the coordinates that will be displayed in the GUI.
	 * Depending on the z parameter the method will know which floor map that should be displayed. 
	 * The method will make sure that a GUI with a map is displayed and the position defined by the input
	 * parameters is marked in the map of the GUI.
	 * 
	 * @param x  X coordinate to display on the map which is an int
	 * @param y  Y coordinate to display on the map which is an int
	 * @param z  Z coordinate to display on the map which is an int
	 */
	public void makeMapGui(int x, int y, int z){
		System.out.println("z = "+z);
		if(z == 1){
			xStartPoint = 1560;
			yStartPoint = 1392;
			displayImage(frame, firstFloorImg, x, y, z);
			System.out.println("Loadet frste etasje");
		}
		else if(z == 2){
			xStartPoint = 1580;
			yStartPoint = 1290;
			displayImage(frame, secondFloorImg, x, y, z);
			System.out.println("Lodet andre etasje");
		}
		frame.setName("Find My Location");
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frame.setVisible(true);
		
	}

	/**
	 * Method for loading in a picture from the hard drive. It takes
	 * in a string giving a reference to the picture that should be loaded.
	 * The method will create a {@link BufferedImage} corresponding to the 
	 * picture file in the hard drive that was loaded.
	 * 
	 * @see BufferedImage
	 * @param ref String giving a reference to the file that will be loaded. 
	 * @return A {@link BufferedImage} created by the file referred to by the
	 * input string 
	 */
	
	public BufferedImage loadImage(String ref) {  
		BufferedImage img = null;  
		try {  
			img = ImageIO.read(new File(ref));  
		} 
		catch (Exception e) {  
			e.printStackTrace();  
		}  
		return img;  
	} 
	
	/**
	 * This method will take a {@link BufferedImage} and load it into a {@link JImagePanel},
	 * and add this JImagePanel into a {@link JFrame}. The input parameters x and y defines 
	 * a position that will be marked in the BufferedImage added to the JImagePanel. The method
	 * will add a {@link JButton} and a {@link JTextField} to the JImagePanel. The JTextField is
	 * used for displaying the input coordinates x, y and z. The JButton is used for calling a 
	 * new location look up.
	 * 
	 * @see JImagePanel
	 * @see JFrame
	 * @see BufferedImage
	 * @see JButton
	 * @see JTextField
	 * @param frame The JFrame that the JImagePanel should be added to.
	 * @param loadpic  The {@link BufferedImage} to load into the {@link JImagePanel}.  
	 * @param x X coordinate for the position an int.
	 * @param y Y coordinate for the position an int.
	 * @param z Z coordinate for the position an int.
	 */
	public void displayImage(JFrame frame, BufferedImage loadpic, int x, int y, int z) {
		 if (panel !=null) {
			frame.remove(panel);
		}
		 BufferedImage printPic = markPosition(loadpic, (xStartPoint-(pixMeter*x))/2,(yStartPoint-(pixMeter*y))/2);
		 System.out.println("Bildets lengde = "+printPic.getWidth());
         frame.setBounds(0, 0, printPic.getWidth(), printPic.getHeight());
         panel = new JImagePanel(printPic, 0, 0);
         panel.add(btn,JImagePanel.CENTER_ALIGNMENT);
         panel.add(coords,JImagePanel.RIGHT_ALIGNMENT);
         coords.setText("X coordinate = "+y+" Y coordinate = "+x+" Z coordinate = "+ z);
         coords.setEditable(false);
         coords.setVisible(true);
         frame.add(panel,BorderLayout.CENTER);
         frame.setLocation(500, 0);
         btn.removeActionListener(listner);
         btn.addActionListener(listner);
         System.out.println("GJORT GUI FERDIG");
	}
	/**
	 * Method used for resizeing a {@link BufferedImage}. The BufferedImage will 
	 * be resize with height = newH and width = newW. 
	 * 
	 * @see BufferedImage
	 * @param img The BufferedImage to resize.
	 * @param newW The width that is wanted on the BufferedImage an int.
	 * @param newH The height that is wanted on the BufferedImage an int.
	 * @return The resized image. 
	 */
	public BufferedImage reSize(BufferedImage img, int newW, int newH) {  
         int w = img.getWidth();  
         int h = img.getHeight();  
         BufferedImage reSizeImg = new BufferedImage(newW, newH,img.getType());  
         Graphics2D g = reSizeImg.createGraphics();  
         g.setRenderingHint(RenderingHints.KEY_INTERPOLATION, RenderingHints.VALUE_INTERPOLATION_BILINEAR);  
         g.drawImage(img, 0, 0, newW, newH, 0, 0, w, h, null);  
         g.dispose();  
         return reSizeImg;  
         }
	
	/**
	 * Method used for marking a position on a {@link BufferedImage}.
	 * The position will be marked with and oval shape within 5*5 pixels. 
	 * 
	 * @see BufferedImage
	 * @param img The BufferedImage that should be marked.
	 * @param x An int describing the X coordinate of the position that should be marked.
	 * @param y An int describing the Y coordinate of the position that should be marked.
	 * @return A BufferedImage with the marked position.
	 */
	public BufferedImage markPosition(BufferedImage img, int x, int y){
		WritableRaster raster = img.copyData( null );
		BufferedImage copy = new BufferedImage( img.getColorModel(), raster, img.isAlphaPremultiplied(), null );
		Graphics2D g = (Graphics2D)copy.getGraphics();
		g.setColor(Color.red);
		g.fillOval(x, y, 5, 5);
		return copy;
	}

	public JButton getBtn() {
		return btn;
	}

	public BufferedImage getFirstFloorImg() {
		return firstFloorImg;
	}

	public void setFirstFloorImg(BufferedImage firstFloorImg) {
		this.firstFloorImg = firstFloorImg;
	}

	public BufferedImage getSecondFloorImg() {
		return secondFloorImg;
	}

	public void setSecondFloorImg(BufferedImage secondFloorImg) {
		this.secondFloorImg = secondFloorImg;
	}

	public String getFirstFloor() {
		return firstFloor;
	}

	public void setFirstFloor(String firstFloor) {
		this.firstFloor = firstFloor;
	}

	public String getSecondFloor() {
		return secondFloor;
	}

	public void setSecondFloor(String secondFloor) {
		this.secondFloor = secondFloor;
	}

	public JButton getStartBtn() {
		return startBtn;
	}

	public void setStartBtn(JButton startBtn) {
		this.startBtn = startBtn;
	}

	public void setBtn(JButton btn) {
		this.btn = btn;
	}

	public JFrame getStartFrame() {
		return startFrame;
	}
	

}
