package Location;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.image.BufferedImage;
import java.io.IOException;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;

/**
 * @author	Gerhard Wiese
 * 
 * 
 * The MainSystem class is the core of the location system. 
 * Having instances of the different components used in the system 
 *
 */
public class MainSystem implements ActionListener {

static DataBaseReader dbReader;	
static NetworkReader netReader;
static Calculator calc;
//static Gui gui;
static RegisterLocationGui regGui;
public static ArrayList<AccessPoint> accessPoints;
static long time;

	/**
	 * Constructor of the MainSystem class initializing the different components
	 * of the location system: {@link Gui}, {@link DataBaseReader}, {@link NetworkReader} and {@link Calculator}
	 * 
	 * @see DataBaseReader
	 * @see NetworkReader
	 * @see Gui
	 * @see Calculator
	 */
	public MainSystem(){
		regGui = new RegisterLocationGui(this);
		dbReader = new DataBaseReader();
		netReader = new NetworkReader();
		accessPoints = new ArrayList<AccessPoint>();
		calc = new Calculator();
		regGui.makeGui();
		//dbReader.connectToDB();
		//gui.startUpGui();
	}


	/**
	 * This method starts up the location system, by calling the 
	 * constructor of the MainSystem class.
	 * 
	 * @param args
	 */
	public static void main( String [  ]  args ) {
			new MainSystem();
			

	}


	/* (non-Javadoc)
	 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 */
/*	public void actionPerformed(ActionEvent e) {
		if(e.getActionCommand().equals(gui.getBtn().getActionCommand())){
			System.out.println("BUTTON PRESSED");		
			time = System.currentTimeMillis();
			try {
				dbReader.findLocation();
			} catch (SQLException e1) {
				// TODO Auto-generated catch block
				e1.printStackTrace();
			} catch (IOException e1) {
				// TODO Auto-generated catch block
				e1.printStackTrace();
			} catch (InterruptedException e1) {
				// TODO Auto-generated catch block
				e1.printStackTrace();
			}
			gui.makeMapGui(dbReader.getYcoord(), dbReader.getXcoord(), dbReader.getZcoord());
			long gui = (System.currentTimeMillis() - dbReader.euclid);
			System.out.println("Printe gui tok"+gui+" millisek");
			long searchTime = (System.currentTimeMillis() - time);
			System.out.println("Sket tok "+(searchTime)+" millisek");
			//gui.getBtn().setEnabled(false);
		}
		else if(e.getActionCommand().equals(gui.getStartBtn().getActionCommand())){
			System.out.println("Start button ble trykket !!");
			try {
				dbReader.findLocation();
			} catch (SQLException e1) {
				// TODO Auto-generated catch block
				e1.printStackTrace();
			} catch (IOException e1) {
				// TODO Auto-generated catch block
				e1.printStackTrace();
			} catch (InterruptedException e1) {
				// TODO Auto-generated catch block
				e1.printStackTrace();
			}
			BufferedImage img  = gui.loadImage(gui.getFirstFloor());
			img = gui.reSize(img, img.getWidth()/2, img.getHeight()/2);
			gui.setFirstFloorImg(img);
			BufferedImage img2 = gui.loadImage(gui.getSecondFloor());
			img2 = gui.reSize(img2, img2.getWidth()/2, img2.getHeight()/2);
			gui.setSecondFloorImg(img2);
			gui.makeMapGui(dbReader.getYcoord(), dbReader.getXcoord(), dbReader.getZcoord());
			gui.getStartFrame().setVisible(false);
			gui.getStartFrame().remove(gui.getStartBtn());
		}
	}


	public void keyPressed(KeyEvent e) {
		if(e.getKeyCode() == KeyEvent.VK_ENTER){
			gui.getBtn().doClick();
		}
	}


	public void keyReleased(KeyEvent e) {
		// TODO Auto-generated method stub
		
	}


	public void keyTyped(KeyEvent e) {
		// TODO Auto-generated method stub
		
	}


	public ArrayList<AccessPoint> getAccessPoints() {
		return accessPoints;
	}

*/
	//METODE BENYTTET TIL GUI FOR  LEGGE INN MYE PUNKTER I SYSTEMET. 
	
	public void actionPerformed(ActionEvent e){
		if(e.getActionCommand().equals(regGui.getBtn().getActionCommand())){
			if(regGui.xcoord.getText().length() ==0){
				regGui.result.setText("Input x cordinate");
			}
			else if(regGui.ycoord.getText().length() ==0){
				regGui.result.setText("Input y cordinate");
			}
			else if(regGui.zcoord.getText().length() ==0){
				regGui.result.setText("Input z cordinate");
			}
			else{
			try {
				MainSystem.netReader.readInInfo();
			} catch (IOException e1) {
				// TODO Auto-generated catch block
				e1.printStackTrace();
			} catch (InterruptedException e1) {
				// TODO Auto-generated catch block
				e1.printStackTrace();
			}
			MainSystem.calc.sortAccessPointsByRssi();
			int x = Integer.parseInt(regGui.xcoord.getText());
			int y = Integer.parseInt(regGui.ycoord.getText());
			int z = Integer.parseInt(regGui.zcoord.getText());
			try {
				MainSystem.dbReader.regLoc(x, y, z);
			} catch (SQLException e1) {
				// TODO Auto-generated catch block
				e1.printStackTrace();
			}
			}
		}
	}


}
