package Location;

import java.io.BufferedInputStream;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;
import java.lang.reflect.Array;
import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.io.InputStream;

/**
 * @author Gerhard Wiese
 * 
 * The NetworkReader class handles on contact with
 * the network card for the location system.
 *
 */
public class NetworkReader implements Serializable{
	private String networkName = "eduroam";
	private String macaddress = ".*(([0-9a-fA-F]{2}[-:]){5}[0-9a-fA-F]{1,2})";
	private String rssi = "\\s\\-(\\d{2})";
	private String channel ="\\s{2}(\\d{1,2})\\s{3,}";
	private Pattern networkPattern = Pattern.compile(networkName+macaddress+rssi+channel);
	private Matcher networkMatcher;
	private static String execute = "airport -s";

	/**
	 * The method will read in information about wireless network accesspoint 
	 * reachable by the user equipment. This is done by doing a call to the 
	 * terminal of the equipment. So far this is only implemented for Machentosh 
	 * computers. The method will collect the MAC address(bssid), the corresponding 
	 * network name(ssid) and signal strength(rssi) for each accessible accesspoint.
	 * This information will be stored as {@link AccessPoint} objects in a {@link ArrayList}
	 * called accessPoints in the {@link MainSystem} of the location package. 
	 * 
	 * @see MainSystem
	 * @see AccessPoint
	 * @see ArrayList
	 * @throws IOException
	 * @throws InterruptedException
	 */
	public void readInInfo() throws IOException, InterruptedException{
		System.out.println("starter innlesning");
		MainSystem.accessPoints.clear();
		Runtime run = Runtime.getRuntime();
		Process p = run.exec(execute);
		p.waitFor();
		BufferedReader buf = new BufferedReader(new InputStreamReader(p.getInputStream()));
		String line; 
		line = buf.readLine();
		
		while ((line=buf.readLine())!=null) {
			networkMatcher = networkPattern.matcher(line);			
	        if(networkMatcher.find()){
	        	int channel = Integer.parseInt(networkMatcher.group(4));
	        	System.out.println("Channel = "+channel);
	        	if(channel < 12){
	        		AccessPoint accessPoint = new AccessPoint();
		        	int rssi;	
		        	accessPoint.setSsid(networkName);
		        	accessPoint.setBssi(networkMatcher.group(1));
		        	rssi = Integer.parseInt(networkMatcher.group(3));
		        	System.out.println(rssi);
		        	System.out.println(networkMatcher.group(1));
		        	accessPoint.setRssi(rssi);
		        	MainSystem.accessPoints.add(accessPoint); 
	        	}	        	  	
			}  
		}

	}

	public String getNetworkName() {
		return networkName;
	}

	public void setNetworkName(String networkName) {
		this.networkName = networkName;
	}
}
