//
//  WiFiReader.m
//  Location
//
//  Created by Gerhard Wiese on 19.02.10.
//  Copyright 2010 NTNU. All rights reserved.
//

#import "WiFiReader.h"
#import "AccessPoint.h"

@implementation WiFiReader
- (id)init{
	self = [super init];
	networks = [[NSMutableDictionary alloc] init];
	libHandle = dlopen("/System/Library/SystemConfiguration/WiFiManager.bundle/WiFiManager", RTLD_LAZY);
	char *error;
	if (libHandle == NULL && (error = dlerror()) != NULL)  {
		NSLog(@"%c",error);
		exit(1);
	}
	apple80211Open = dlsym(libHandle, "Apple80211Open");
	apple80211Bind = dlsym(libHandle, "Apple80211BindToInterface");
	apple80211Close = dlsym(libHandle, "Apple80211Close");
	apple80211Scan = dlsym(libHandle, "Apple80211Scan");
	apple80211Open(&airportHandle);
	apple80211Bind(airportHandle, @"en0");  
	return self;
}
- (NSDictionary *)network:(NSString *) BSSID{	
	return [networks objectForKey:@"BSSID"];
}

- (NSDictionary *)networks{
	return networks;
}

- (void)scanNetworks{
	//satt inn ny kode linje her sjekk om den hjelper !!!
	[networks removeAllObjects];
	NSLog(@"Scanning WiFi Channels...");
	NSDictionary *parameters = [[NSDictionary alloc] init];
	NSArray *scan_networks; //is a CFArrayRef of CFDictionaryRef(s) containing key/value data on each discovered network
	apple80211Scan(airportHandle, &scan_networks, parameters);
	for (int i = 0; i < [scan_networks count]; i++) {
		[networks setObject:[scan_networks objectAtIndex: i] forKey:[[scan_networks objectAtIndex: i] objectForKey:@"BSSID"]];
	}
	NSLog(@"Scanning WiFi Channels Finished."); 
}

- (int)numberOfNetworks{
	return [networks count];
}

- ( NSMutableArray * ) getNetworksInfo {
	NSMutableArray * accessPoints = [[NSMutableArray alloc] init];
	for (id key in networks){
		NSString  *networkName = [[networks objectForKey: key] objectForKey:@"SSID_STR"]; 
		if([networkName isEqualToString: @"eduroam"]){
			AccessPoint *acc = [[AccessPoint alloc] init];
			acc.SSID = [[networks objectForKey: key] objectForKey:@"SSID_STR"];
			NSString * bs = [self changeBssidFormat:key];
			acc.BSSID = bs;
			NSString * rssiString = [[networks objectForKey: key] objectForKey:@"RSSI"];
			int rssi = -[rssiString intValue];
			[acc RSSI:rssi];
			[accessPoints addObject:acc];
			[acc release];
		}
	}	
	return accessPoints;
}

-(NSString *) changeBssidFormat:(NSString *) bssid{
	NSString * finBssid = [[NSString alloc] init]; 
	NSArray *split = [bssid componentsSeparatedByString:@":"];
	for(int i= 0; i < [split count]; i++){
		if(([[split objectAtIndex:i] length] < 2)&&(i ==([split count] -1))){
			NSString *applyString = [[NSString alloc] initWithString:@"0"];
			applyString = [applyString stringByAppendingString:[split objectAtIndex:i]];
			finBssid = [finBssid stringByAppendingString:applyString];
		}
		else if([[split objectAtIndex:i] length] < 2){
			NSString *applyString = [[NSString alloc] initWithString:@"0"];
			applyString = [applyString stringByAppendingString:[split objectAtIndex:i]];
			applyString = [applyString stringByAppendingString:@":"];
			finBssid = [finBssid stringByAppendingString:applyString];
		}
		else if(i ==([split count] -1)){
			NSString *applyString = [split objectAtIndex:i];
			finBssid = [finBssid stringByAppendingString:applyString];
		}
		else {
			NSString *applyString = [split objectAtIndex:i];
			applyString = [applyString stringByAppendingString:@":"];
			finBssid = [finBssid stringByAppendingString:applyString];
		}

	}
	
	return finBssid;
}

- (NSMutableArray *) sortAccessPoints:(NSMutableArray*) AccessPoints{
	NSLog(@"Inside sort");
	NSMutableArray *APs = AccessPoints;
	NSLog(@"antall aksesspunkt = %d",[APs count]);
	for(int i = 1; i< [APs count]; i++){
		for(int j = 0; j< i; j++){
			if([[APs objectAtIndex:i] RSSI]  < [[APs objectAtIndex:j] RSSI]){					
				[APs insertObject:[APs objectAtIndex:i] atIndex: j];
				[APs removeObjectAtIndex:i+1];
				break;
			}
			
		}
	}
	NSLog(@"Rett før vi returnerer");
	return APs;	
}

- (void) dealloc {
	apple80211Close(airportHandle);
	[super dealloc];
}
@end
